@@ -99,6 +99,8 @@ __log_manager__ = None
 def set_log_destination(
     use_stdout=False,
     use_disk=False,
+    no_fork=False,
+    stdout_severity="NOTSET",
     logs_directory=None,
     agent_log_file_path="agent.log",
     agent_debug_log_file_suffix="_debug",
@@ -134,6 +136,9 @@ def set_log_destination(
 
     @param use_stdout: True if the logs should be sent to standard out.  If this is False, then use_disk must be true.
     @param use_disk:  True if the logs should be sent to disk.  If this is False, then use_stdout must be true.
+    @param no_fork:  True if we are running in --no-fork mode, logs above a configured severity threshold will be
+        written to stdout.
+    @param stdout_severity: Logs at or above this severity level will be written to stdout if no_fork is true.
     @param logs_directory:  The path of the directory to, by default, write log files.
     @param agent_log_file_path: If not None, then the file path where the main agent log file should be written
         using a RotatingLogHandler scheme, with parameters specified by max_bytes and backup_count.  If the path is
@@ -153,6 +158,8 @@ def set_log_destination(
     __log_manager__.set_log_destination(
         use_stdout=use_stdout,
         use_disk=use_disk,
+        no_fork=no_fork,
+        stdout_severity=stdout_severity,
         logs_directory=logs_directory,
         agent_log_file_path=agent_log_file_path,
         agent_debug_log_file_suffix=agent_debug_log_file_suffix,
@@ -210,6 +217,13 @@ def alternateCurrentFrame():
     return sys._getframe(3)
 
 
+def close_handlers():
+    root_logger = logging.getLogger()
+    for handler in list(root_logger.handlers):
+        root_logger.removeHandler(handler)
+        handler.close()
+
+
 if hasattr(sys, "_getframe"):
     currentframe = alternateCurrentFrame  # NOQA
 # done filching
@@ -1008,27 +1022,39 @@ class RateLimiterLogFilter(object):
             return False
 
 
-class ForceStdoutFilter(object):
-    """A filter that includes any record if it has `force_stdout` as True
+class StdoutFilter(object):
+    """A filter for outputting logs to stdout.
+    Will output the record if `no_fork` is true and the record has a severity of `stdout_severity` or higher.
+    Alternatively if `force_stdout` is true in the record it will always be output to stdout.
     """
 
-    def __init__(self):
+    def __init__(self, no_fork, stdout_severity):
         """Initializes the filter.
         """
+        self.__no_fork = no_fork
+        self.__stdout_severity = getattr(logging, stdout_severity, 0)
 
     def filter(self, record):
         """Performs the filtering.
 
+        We have a partial copy of the filtering done in AgentLogFilter in this one, because we want the same filtering
+        as a normal agent log, but without the separation into debug and non-debug files.
+
         @param record: The record to filter.
         @type record: logging.LogRecord
 
         @return:  True if the record should be logged by this handler.
         @rtype: bool
         """
-        return getattr(record, "force_stdout", False)
+        return getattr(record, "force_stdout", False) or (
+            self.__no_fork
+            and record.levelno >= self.__stdout_severity
+            and getattr(record, "agent_logger", False)
+            and record.metric_log_for_monitor is None
+        )
 
 
-class ForceStderrFilter(object):
+class StderrFilter(object):
     """A filter that includes any record if it has `force_stderr` as True
     """
 
@@ -1404,6 +1430,11 @@ class AgentLogManager(object):
         # If True, then logging will be sent to stdout rather than the file names mentioned above.
         self.__use_stdout = True
 
+        # If True we are running unforked and should log things to stdout as well as the log file, messages with
+        # severity greater than or equal to `self.__stdout_severity` will be logged to stdout.
+        self.__no_fork = False
+        self.__stdout_severity = "NOTSET"
+
         # If using a rotating log, this is the maximum number of bytes that can be written before it is rotated.
         self.__rotation_max_bytes = 20 * 1024 * 1024
         # The number of previous logs that will be kept from the rotation.
@@ -1435,6 +1466,8 @@ class AgentLogManager(object):
         self,
         use_stdout=False,
         use_disk=False,
+        no_fork=False,
+        stdout_severity="NOTSET",
         logs_directory=None,
         agent_log_file_path="agent.log",
         agent_debug_log_file_suffix="_debug",
@@ -1450,6 +1483,8 @@ class AgentLogManager(object):
             raise Exception("You must specify at least one of use_stdout or use_diskk.")
 
         self.__use_stdout = use_stdout
+        self.__no_fork = no_fork
+        self.__stdout_severity = stdout_severity
         self.__rotation_max_bytes = max_bytes
         self.__rotation_backup_count = backup_count
         self.__log_write_rate = log_write_rate
@@ -1606,21 +1641,21 @@ class AgentLogManager(object):
         # Create the right type of handler.
         if is_force_stdout:
             handler = logging.StreamHandler(sys.stdout)
-            handler.addFilter(ForceStdoutFilter())
+            handler.addFilter(StdoutFilter(self.__no_fork, self.__stdout_severity))
         elif is_force_stderr:
             handler = logging.StreamHandler(sys.stderr)
-            handler.addFilter(ForceStderrFilter())
+            handler.addFilter(StderrFilter())
         elif self.__use_stdout:
             handler = logging.StreamHandler(sys.stdout)
+            handler.addFilter(AgentLogFilter(is_debug))
         else:
             handler = logging.handlers.RotatingFileHandler(
                 file_path,
                 maxBytes=self.__rotation_max_bytes,
                 backupCount=self.__rotation_backup_count,
             )
+            handler.addFilter(AgentLogFilter(is_debug))
 
-        # Limit to only emitting the write log records.
-        handler.addFilter(AgentLogFilter(is_debug))
         formatter = AgentLogFormatter()
         # Rate limit the log if this is the main log since we are copying it up to Scalyr as well.
         if not is_debug:
