@@ -15,21 +15,23 @@
 #
 # author: Steven Czerwinski <czerwin@scalyr.com>
 
-__author__ = 'czerwin@scalyr.com'
+
+from __future__ import absolute_import
+
+__author__ = "czerwin@scalyr.com"
 
 import os
-import unittest
 
 from scalyr_agent.__scalyr__ import get_install_root, get_package_root, SCALYR_VERSION
+from scalyr_agent.test_base import ScalyrTestCase
 
 
-class TestUtil(unittest.TestCase):
-
+class TestUtil(ScalyrTestCase):
     def test_version(self):
-        self.assertTrue(SCALYR_VERSION.startswith('2.'))
+        self.assertTrue(SCALYR_VERSION.startswith("2."))
 
     def test_get_install_root(self):
-        self.assertEquals(os.path.basename(get_install_root()), 'scalyr-agent-2')
+        self.assertEquals(os.path.basename(get_install_root()), "scalyr-agent-2")
 
     def test_get_package_root(self):
-        self.assertEquals(os.path.basename(get_package_root()), 'scalyr_agent')
\ No newline at end of file
+        self.assertEquals(os.path.basename(get_package_root()), "scalyr_agent")
