@@ -15,18 +15,46 @@
 #
 # author: Steven Czerwinski <czerwin@scalyr.com>
 
-__author__ = 'czerwin@scalyr.com'
+from __future__ import unicode_literals
+from __future__ import absolute_import
+from io import open
 
-import unittest
-import cStringIO
+__author__ = "czerwin@scalyr.com"
 
-from scalyr_agent.agent_status import OverallStats, AgentStatus, ConfigStatus, LogProcessorStatus, MonitorStatus
-from scalyr_agent.agent_status import CopyingManagerStatus, MonitorManagerStatus, LogMatcherStatus, report_status
+import io
+import os
+import json
+import tempfile
+
+import mock
+import six
+
+from scalyr_agent.agent_main import ScalyrAgent
+from scalyr_agent.agent_main import STATUS_FORMAT_FILE
+from scalyr_agent.configuration import Configuration
+from scalyr_agent.platform_controller import PlatformController
+from scalyr_agent.agent_status import (
+    OverallStats,
+    AgentStatus,
+    ConfigStatus,
+    LogProcessorStatus,
+    MonitorStatus,
+)
+from scalyr_agent.agent_status import (
+    CopyingManagerStatus,
+    MonitorManagerStatus,
+    LogMatcherStatus,
+    report_status,
+)
+
+from scalyr_agent.test_base import ScalyrTestCase
+from scalyr_agent.compat import os_environ_unicode
+
+BASE_DIR = os.path.abspath(os.path.dirname(os.path.abspath(__file__)))
 
 
-class TestOverallStats(unittest.TestCase):
-
-    def test_read_file_as_json(self):
+class TestOverallStats(ScalyrTestCase):
+    def test___add___method(self):
         a = OverallStats()
         b = OverallStats()
 
@@ -42,6 +70,7 @@ class TestOverallStats(unittest.TestCase):
         a.total_requests_sent = 1
         a.total_requests_failed = 2
         a.total_request_bytes_sent = 3
+        a.total_compressed_request_bytes_sent = 1
         a.total_response_bytes_received = 4
         a.total_request_latency_secs = 5
         a.total_connections_created = 6
@@ -58,6 +87,7 @@ class TestOverallStats(unittest.TestCase):
         b.total_requests_sent = 7
         b.total_requests_failed = 8
         b.total_request_bytes_sent = 9
+        b.total_compressed_request_bytes_sent = 4
         b.total_response_bytes_received = 10
         b.total_request_latency_secs = 11
         b.total_connections_created = 12
@@ -76,36 +106,45 @@ class TestOverallStats(unittest.TestCase):
         self.assertEquals(c.total_requests_sent, 8)
         self.assertEquals(c.total_requests_failed, 10)
         self.assertEquals(c.total_request_bytes_sent, 12)
+        self.assertEquals(c.total_compressed_request_bytes_sent, 5)
         self.assertEquals(c.total_response_bytes_received, 14)
         self.assertEquals(c.total_request_latency_secs, 16)
         self.assertEquals(c.total_connections_created, 18)
 
 
-class TestReportStatus(unittest.TestCase):
+class TestReportStatus(ScalyrTestCase):
+    def tearDown(self):
+        os.environ.clear()
+        os.environ.update(self.saved_env)
+
     def setUp(self):
+        super(TestReportStatus, self).setUp()
+        self.saved_env = dict((k, v) for k, v in six.iteritems(os_environ_unicode))
+        os.environ.clear()
         self.time = 1409958853
         self.status = AgentStatus()
         self.status.launch_time = self.time - 86400
-        self.status.log_path = '/var/logs/scalyr-agent/agent.log'
-        self.status.scalyr_server = 'https://agent.scalyr.com'
-        self.status.server_host = 'test_machine'
-        self.status.user = 'root'
-        self.status.version = '2.0.0.beta.7'
+        self.status.log_path = "/var/logs/scalyr-agent/agent.log"
+        self.status.scalyr_server = "https://agent.scalyr.com"
+        self.status.server_host = "test_machine"
+        self.status.user = "root"
+        self.status.version = "2.0.0.beta.7"
+        self.status.python_version = "3.6.8"
 
         config_status = ConfigStatus()
         self.status.config_status = config_status
         config_status.last_read_time = self.time - 43200
         config_status.last_check_time = self.time
         config_status.last_good_read = self.time - 43000
-        config_status.path = '/etc/scalyr-agent-2/agent.json'
-        config_status.status = 'Good'
-        config_status.additional_paths = ['/etc/scalyr-agent-2/agent.d/server.json']
+        config_status.path = "/etc/scalyr-agent-2/agent.json"
+        config_status.status = "Good"
+        config_status.additional_paths = ["/etc/scalyr-agent-2/agent.d/server.json"]
 
         copying_status = CopyingManagerStatus()
         self.status.copying_manager_status = copying_status
         copying_status.last_attempt_size = 10000
         copying_status.last_attempt_time = self.time - 60
-        copying_status.last_response_status = 'success'
+        copying_status.last_response_status = "success"
         copying_status.total_errors = 0
         copying_status.total_bytes_uploaded = 10000
         copying_status.last_success_time = self.time - 60
@@ -115,17 +154,17 @@ class TestReportStatus(unittest.TestCase):
         copying_status.log_matchers.append(log_matcher)
         log_matcher.is_glob = False
         log_matcher.last_check_time = self.time - 10
-        log_matcher.log_path = '/var/logs/tomcat6/access.log'
+        log_matcher.log_path = "/var/logs/tomcat6/access.log"
 
         # Add in another matcher that isn't a glob but does have a match.
         log_matcher = LogMatcherStatus()
         copying_status.log_matchers.append(log_matcher)
         log_matcher.is_glob = False
         log_matcher.last_check_time = self.time - 10
-        log_matcher.log_path = '/var/logs/tomcat6/catalina.log'
+        log_matcher.log_path = "/var/logs/tomcat6/catalina.log"
         process_status = LogProcessorStatus()
         log_matcher.log_processors_status.append(process_status)
-        process_status.log_path = '/var/logs/tomcat6/catalina.log'
+        process_status.log_path = "/var/logs/tomcat6/catalina.log"
         process_status.last_scan_time = self.time - 120
         process_status.total_bytes_copied = 2341234
         process_status.total_bytes_pending = 1243
@@ -141,10 +180,10 @@ class TestReportStatus(unittest.TestCase):
         copying_status.log_matchers.append(log_matcher)
         log_matcher.is_glob = True
         log_matcher.last_check_time = self.time - 10
-        log_matcher.log_path = '/var/logs/cron/*.log'
+        log_matcher.log_path = "/var/logs/cron/*.log"
         process_status = LogProcessorStatus()
         log_matcher.log_processors_status.append(process_status)
-        process_status.log_path = '/var/logs/cron/logrotate.log'
+        process_status.log_path = "/var/logs/cron/logrotate.log"
         process_status.last_scan_time = self.time - 120
         process_status.total_bytes_copied = 2341234
         process_status.total_bytes_pending = 1243
@@ -156,7 +195,7 @@ class TestReportStatus(unittest.TestCase):
         process_status.total_redactions = 0
         process_status = LogProcessorStatus()
         log_matcher.log_processors_status.append(process_status)
-        process_status.log_path = '/var/logs/cron/ohno.log'
+        process_status.log_path = "/var/logs/cron/ohno.log"
         process_status.last_scan_time = self.time - 120
         process_status.total_bytes_copied = 23434
         process_status.total_bytes_pending = 12943
@@ -172,7 +211,7 @@ class TestReportStatus(unittest.TestCase):
         copying_status.log_matchers.append(log_matcher)
         log_matcher.is_glob = True
         log_matcher.last_check_time = self.time - 10
-        log_matcher.log_path = '/var/logs/silly/*.log'
+        log_matcher.log_path = "/var/logs/silly/*.log"
 
         # Now for the monitors.
         monitor_manager = MonitorManagerStatus()
@@ -182,26 +221,39 @@ class TestReportStatus(unittest.TestCase):
         monitor_status = MonitorStatus()
         monitor_manager.monitors_status.append(monitor_status)
         monitor_status.is_alive = True
-        monitor_status.monitor_name = 'linux_process_metrics(agent)'
+        monitor_status.monitor_name = "linux_process_metrics(agent)"
         monitor_status.reported_lines = 50
         monitor_status.errors = 2
 
         monitor_status = MonitorStatus()
         monitor_manager.monitors_status.append(monitor_status)
         monitor_status.is_alive = True
-        monitor_status.monitor_name = 'linux_system_metrics()'
+        monitor_status.monitor_name = "linux_system_metrics()"
         monitor_status.reported_lines = 20
         monitor_status.errors = 0
 
         monitor_status = MonitorStatus()
         monitor_manager.monitors_status.append(monitor_status)
         monitor_status.is_alive = False
-        monitor_status.monitor_name = 'bad_monitor()'
+        monitor_status.monitor_name = "bad_monitor()"
         monitor_status.reported_lines = 20
         monitor_status.errors = 40
 
     def test_basic(self):
-        output = cStringIO.StringIO()
+        output = io.StringIO()
+
+        # Environment variables
+        os.environ["SCALYR_API_KEY"] = "This private key should be redacted"
+        # intentionally leave out required scalyr_server
+        os.environ["scalyr_K8S_CLUSTER_NAME"] = "test_cluster"
+        os.environ[
+            "K8S_EVENT_DISABLE"
+        ] = "Special-case-included despite missing prefix.  Appears at end of main keys."
+        os.environ["SCALYR_K8S_EVENT_DISABLE"] = "true"
+        os.environ["SCALYR_AAA"] = "Should appear just after main keys"
+        os.environ["SCALYR_XXX_A"] = "A before b (ignores case)"
+        os.environ["sCaLyR_XXX_b"] = "b after A (ignores case)"
+        os.environ["SCALYR_ZZZ"] = "Should appear at the end"
         report_status(output, self.status, self.time)
 
         expected_output = """Scalyr Agent status.  See https://www.scalyr.com/help/scalyr-agent-2 for help
@@ -209,6 +261,7 @@ class TestReportStatus(unittest.TestCase):
 Current time:     Fri Sep  5 23:14:13 2014 UTC
 Agent started at: Thu Sep  4 23:14:13 2014 UTC
 Version:          2.0.0.beta.7
+Python version:   3.6.8
 Agent running as: root
 Agent log:        /var/logs/scalyr-agent/agent.log
 ServerHost:       test_machine
@@ -225,6 +278,16 @@ Status:                Good (files parsed successfully)
 Last checked:          Fri Sep  5 23:14:13 2014 UTC
 Last changed observed: Fri Sep  5 11:14:13 2014 UTC
 
+Environment variables: SCALYR_API_KEY = <Redacted>
+                       SCALYR_SERVER = <Missing>
+                       K8S_EVENT_DISABLE = Special-case-included despite missing prefix.  Appears at end of main keys.
+                       SCALYR_AAA = Should appear just after main keys
+                       scalyr_K8S_CLUSTER_NAME = test_cluster
+                       SCALYR_K8S_EVENT_DISABLE = true
+                       SCALYR_XXX_A = A before b (ignores case)
+                       sCaLyR_XXX_b = b after A (ignores case)
+                       SCALYR_ZZZ = Should appear at the end
+
 
 Log transmission:
 =================
@@ -260,9 +323,9 @@ Failed monitors:
         self.assertEquals(expected_output, output.getvalue())
 
     def test_bad_config(self):
-        self.status.config_status.last_error = 'Bad stuff'
+        self.status.config_status.last_error = "Bad stuff"
 
-        output = cStringIO.StringIO()
+        output = io.StringIO()
         report_status(output, self.status, self.time)
 
         expected_output = """Scalyr Agent status.  See https://www.scalyr.com/help/scalyr-agent-2 for help
@@ -270,6 +333,7 @@ Failed monitors:
 Current time:     Fri Sep  5 23:14:13 2014 UTC
 Agent started at: Thu Sep  4 23:14:13 2014 UTC
 Version:          2.0.0.beta.7
+Python version:   3.6.8
 Agent running as: root
 Agent log:        /var/logs/scalyr-agent/agent.log
 ServerHost:       test_machine
@@ -287,6 +351,9 @@ Last checked:          Fri Sep  5 23:14:13 2014 UTC
 Last changed observed: Fri Sep  5 11:14:13 2014 UTC
 Parsing error:         Bad stuff
 
+Environment variables: SCALYR_API_KEY = <Missing>
+                       SCALYR_SERVER = <Missing>
+
 
 Log transmission:
 =================
@@ -322,11 +389,11 @@ Failed monitors:
         self.assertEquals(expected_output, output.getvalue())
 
     def test_bad_copy_response(self):
-        self.status.copying_manager_status.last_response = 'Some weird stuff'
-        self.status.copying_manager_status.last_response_status = 'error'
+        self.status.copying_manager_status.last_response = "Some weird stuff"
+        self.status.copying_manager_status.last_response_status = "error"
         self.status.copying_manager_status.total_errors = 5
 
-        output = cStringIO.StringIO()
+        output = io.StringIO()
         report_status(output, self.status, self.time)
 
         expected_output = """Scalyr Agent status.  See https://www.scalyr.com/help/scalyr-agent-2 for help
@@ -334,6 +401,7 @@ Failed monitors:
 Current time:     Fri Sep  5 23:14:13 2014 UTC
 Agent started at: Thu Sep  4 23:14:13 2014 UTC
 Version:          2.0.0.beta.7
+Python version:   3.6.8
 Agent running as: root
 Agent log:        /var/logs/scalyr-agent/agent.log
 ServerHost:       test_machine
@@ -350,6 +418,9 @@ Status:                Good (files parsed successfully)
 Last checked:          Fri Sep  5 23:14:13 2014 UTC
 Last changed observed: Fri Sep  5 11:14:13 2014 UTC
 
+Environment variables: SCALYR_API_KEY = <Missing>
+                       SCALYR_SERVER = <Missing>
+
 
 Log transmission:
 =================
@@ -386,4 +457,114 @@ Running monitors:
 Failed monitors:
   bad_monitor() 20 lines emitted, 40 errors
 """
-        self.assertEquals(expected_output, output.getvalue())
\ No newline at end of file
+        self.assertEquals(expected_output, output.getvalue())
+
+    def test_status_to_dict(self):
+        result = self.status.to_dict()
+
+        # Simple value on the OverallStats object
+        self.assertEqual(result["user"], "root")
+        self.assertEqual(result["version"], "2.0.0.beta.7")
+
+        # Verify nested status objects are recursively serialized to simple native types
+        config_status = result["config_status"]
+        self.assertEqual(config_status["status"], "Good")
+        self.assertEqual(config_status["last_error"], None)
+        self.assertEqual(config_status["last_check_time"], 1409958853)
+
+        copying_manager_status = result["copying_manager_status"]
+        self.assertEqual(copying_manager_status["last_attempt_size"], 10000)
+        self.assertEqual(copying_manager_status["log_matchers"][0]["is_glob"], False)
+        self.assertEqual(
+            copying_manager_status["log_matchers"][0]["log_path"],
+            "/var/logs/tomcat6/access.log",
+        )
+
+        monitor_manager_status = result["monitor_manager_status"]
+        self.assertEqual(monitor_manager_status["total_alive_monitors"], 2)
+        self.assertEqual(monitor_manager_status["monitors_status"][0]["errors"], 2)
+        self.assertEqual(monitor_manager_status["monitors_status"][0]["is_alive"], True)
+        self.assertEqual(
+            monitor_manager_status["monitors_status"][0]["monitor_name"],
+            "linux_process_metrics(agent)",
+        )
+
+        # Verify dict contains only simple types - JSON.dumps would fail if it doesn't
+        result_json = json.dumps(result)
+        self.assertEqual(json.loads(result_json), result)
+
+
+class AgentMainStatusHandlerTestCase(ScalyrTestCase):
+    def setUp(self):
+        super(AgentMainStatusHandlerTestCase, self).setUp()
+
+        self.data_path = tempfile.mkdtemp(suffix="agent-data-path")
+        self.status_format_file = os.path.join(self.data_path, STATUS_FORMAT_FILE)
+
+        default_paths = mock.Mock()
+        default_paths.agent_data_path = self.data_path
+        default_paths.agent_log_path = "agent.log"
+
+        config_file = os.path.join(BASE_DIR, "fixtures/configs/agent1.json")
+        config = Configuration(config_file, default_paths, None)
+        config.parse()
+
+        self.agent = ScalyrAgent(PlatformController())
+        self.agent._ScalyrAgent__config = config
+
+    def test_report_status_to_file_no_format_specified(self):
+        # No format is provided, should default to "text"
+        status_file = self.agent._ScalyrAgent__report_status_to_file()
+        content = self._read_status_file(status_file)
+
+        self.assertTrue("Current time:" in content)
+        self.assertTrue("ServerHost:" in content)
+        self.assertTrue("Agent configuration:" in content)
+
+    def test_report_status_to_file_text_format_specified(self):
+        # "text" format is explicitly provided
+        self._write_status_format_file("text")
+
+        status_file = self.agent._ScalyrAgent__report_status_to_file()
+        content = self._read_status_file(status_file)
+
+        self.assertTrue("Current time:" in content)
+        self.assertTrue("ServerHost:" in content)
+        self.assertTrue("Agent configuration:" in content)
+
+    def test_report_status_to_file_invalid_format_specified(self):
+        # invalid format is explicitly provided, should fall back to text
+        self._write_status_format_file("invalid")
+
+        status_file = self.agent._ScalyrAgent__report_status_to_file()
+        content = self._read_status_file(status_file)
+
+        self.assertTrue("Current time:" in content)
+        self.assertTrue("ServerHost:" in content)
+        self.assertTrue("Agent configuration:" in content)
+
+    def test_report_status_to_file_json_format_specified(self):
+        # "json" format is explicitly provided
+        self._write_status_format_file("json")
+
+        status_file = self.agent._ScalyrAgent__report_status_to_file()
+        content = self._read_status_file(status_file)
+
+        self.assertFalse("Current time:" in content)
+        self.assertFalse("ServerHost:" in content)
+        self.assertFalse("Agent configuration:" in content)
+
+        parsed = json.loads(content)
+        self.assertTrue("config_status" in parsed)
+        self.assertTrue("user" in parsed)
+        self.assertTrue("scalyr_server" in parsed)
+
+    def _write_status_format_file(self, status_format):
+        with open(self.status_format_file, "w") as fp:
+            fp.write(status_format.strip())
+
+    def _read_status_file(self, status_file_path):
+        with open(status_file_path, "r") as fp:
+            content = fp.read()
+
+        return content
