@@ -22,6 +22,7 @@ __author__ = "czerwin@scalyr.com"
 from io import BytesIO
 
 import mock
+import time
 
 from scalyr_agent.__scalyr__ import SCALYR_VERSION
 
@@ -46,7 +47,6 @@ class AddEventsRequestTest(ScalyrTestCase):
     def setUp(self):
         super(AddEventsRequestTest, self).setUp()
         self.__body = {"token": "fakeToken"}
-        scalyr_client._set_last_timestamp(0)
 
     def test_basic_case(self):
         request = AddEventsRequest(self.__body)
@@ -245,6 +245,56 @@ class AddEventsRequestTest(ScalyrTestCase):
         )
         request.close()
 
+    def test_monotonically_increasing_timestamp(self):
+        request = AddEventsRequest(self.__body, enforce_monotonic_timestamps=True)
+        scalyr_client._set_last_timestamp(0)
+
+        ts = 2000
+
+        expected = str(ts + 1)
+
+        self.assertTrue(
+            request.add_event(Event().set_message("eventOne"), timestamp=ts)
+        )
+        self.assertTrue(request.add_event(Event().set_message("eventTwo"), timestamp=1))
+
+        json = test_util.parse_scalyr_request(request.get_payload())
+        event = json["events"][1]
+        self.assertEquals(event["ts"], expected)
+
+    def test_no_monotonically_increasing_timestamp(self):
+        request = AddEventsRequest(self.__body)
+
+        ts = 2000
+
+        self.assertTrue(
+            request.add_event(Event().set_message("eventOne"), timestamp=ts)
+        )
+        self.assertTrue(request.add_event(Event().set_message("eventTwo"), timestamp=1))
+
+        json = test_util.parse_scalyr_request(request.get_payload())
+        event = json["events"][1]
+        self.assertEquals(event["ts"], "1")
+
+    def test_timestamp_none(self):
+        request = AddEventsRequest(self.__body)
+        request.set_client_time(100)
+
+        ts = int(time.time() * 1e9)
+
+        self.assertTrue(
+            request.add_event(Event().set_message("eventOne"), timestamp=None)
+        )
+
+        json = test_util.parse_scalyr_request(request.get_payload())
+        event = json["events"][0]
+        event_ts = int(event["ts"])
+        threshold = abs(event_ts - ts)
+
+        # allow a threshold of 1 second to have elapsed between reading the time.time and
+        # setting the event timestamp in add_event
+        self.assertTrue(threshold < 1e9)
+
     def test_sequence_id_but_no_number(self):
         request = AddEventsRequest(self.__body)
         request.set_client_time(1)
@@ -830,22 +880,41 @@ class PostFixBufferTest(ScalyrTestCase):
         self.assertTrue(
             test_buffer.add_log_and_thread_entry("log_5", "histogram_builder", {})
         )
+
+        # NOTE: Order is important since .length call depends on the cached size
+        content = test_buffer.content(cache_size=True)
+        length = test_buffer.length
+
         self.assertEquals(
-            test_buffer.length, len(test_buffer.content(cache_size=False))
+            length,
+            len(content),
+            "Buffer content: %s" % (test_buffer.content(cache_size=False)),
         )
 
         self.assertTrue(
             test_buffer.add_log_and_thread_entry("log_12", "ok_builder", {})
         )
+
+        content = test_buffer.content(cache_size=True)
+        length = test_buffer.length
+
         self.assertEquals(
-            test_buffer.length, len(test_buffer.content(cache_size=False))
+            length,
+            len(content),
+            "Buffer content: %s" % (test_buffer.content(cache_size=False)),
         )
 
         self.assertTrue(
             test_buffer.add_log_and_thread_entry("log", "histogram_builder_foo", {})
         )
+
+        content = test_buffer.content(cache_size=True)
+        length = test_buffer.length
+
         self.assertEquals(
-            test_buffer.length, len(test_buffer.content(cache_size=False))
+            length,
+            len(content),
+            "Buffer content: %s" % (test_buffer.content(cache_size=False)),
         )
 
         self.assertEquals(
@@ -860,13 +929,20 @@ class PostFixBufferTest(ScalyrTestCase):
         test_buffer = PostFixBuffer(self.__format)
         test_buffer.set_client_timestamp(1)
 
+        # NOTE: Order is important since .length call depends on the cached size
         self.assertTrue(
             test_buffer.add_log_and_thread_entry(
                 "log_5", "histogram_builder", {}, fail_if_buffer_exceeds=1000000
             )
         )
+
+        content = test_buffer.content(cache_size=True)
+        length = test_buffer.length
+
         self.assertEquals(
-            test_buffer.length, len(test_buffer.content(cache_size=False))
+            length,
+            len(content),
+            "Buffer content: %s" % (test_buffer.content(cache_size=False)),
         )
 
         self.assertFalse(
@@ -875,8 +951,13 @@ class PostFixBufferTest(ScalyrTestCase):
             )
         )
 
+        content = test_buffer.content(cache_size=True)
+        length = test_buffer.length
+
         self.assertEquals(
-            test_buffer.length, len(test_buffer.content(cache_size=False))
+            length,
+            len(content),
+            "Buffer content: %s" % (test_buffer.content(cache_size=False)),
         )
 
         self.assertEquals(
@@ -901,8 +982,13 @@ class PostFixBufferTest(ScalyrTestCase):
 
         test_buffer.set_position(position)
 
+        content = test_buffer.content(cache_size=True)
+        length = test_buffer.length
+
         self.assertEquals(
-            test_buffer.length, len(test_buffer.content(cache_size=False))
+            length,
+            len(content),
+            "Buffer content: %s" % (test_buffer.content(cache_size=False)),
         )
 
         self.assertEquals(
@@ -966,6 +1052,51 @@ class ClientSessionTest(BaseScalyrLogCaptureTestCase):
         call_kwargs = session._ScalyrClientSession__connection.post.call_args_list[0][1]
         self.assertEqual(call_kwargs["body"], "compressed")
 
+    @mock.patch("scalyr_agent.scalyr_client.time.time", mock.Mock(return_value=0))
+    def test_send_request_timestamp_doesnt_increases_monotonically(self):
+        session = ScalyrClientSession(
+            "https://dummserver.com", "DUMMY API KEY", SCALYR_VERSION,
+        )
+
+        session._ScalyrClientSession__connection = mock.Mock()
+        session._ScalyrClientSession__receive_response = mock.Mock()
+
+        add_events_request = session.add_events_request()
+
+        ts = 2000
+
+        add_events_request.add_event(Event().set_message("eventOne"), timestamp=ts)
+        add_events_request.add_event(Event().set_message("eventTwo"), timestamp=1)
+
+        json = test_util.parse_scalyr_request(add_events_request.get_payload())
+        event = json["events"][1]
+        self.assertEquals(event["ts"], "1")
+
+    @mock.patch("scalyr_agent.scalyr_client.time.time", mock.Mock(return_value=0))
+    def test_send_request_timestamp_increases_monotonically(self):
+        session = ScalyrClientSession(
+            "https://dummserver.com",
+            "DUMMY API KEY",
+            SCALYR_VERSION,
+            enforce_monotonic_timestamps=True,
+        )
+
+        session._ScalyrClientSession__connection = mock.Mock()
+        session._ScalyrClientSession__receive_response = mock.Mock()
+        scalyr_client._set_last_timestamp(0)
+
+        add_events_request = session.add_events_request()
+
+        ts = 2000
+        expected = str(ts + 1)
+
+        add_events_request.add_event(Event().set_message("eventOne"), timestamp=ts)
+        add_events_request.add_event(Event().set_message("eventTwo"), timestamp=1)
+
+        json = test_util.parse_scalyr_request(add_events_request.get_payload())
+        event = json["events"][1]
+        self.assertEquals(event["ts"], expected)
+
     @mock.patch("scalyr_agent.scalyr_client.time.time", mock.Mock(return_value=0))
     def test_send_request_body_is_logged_raw_uncompressed_long_body_is_truncated(self):
         # Verify that very large bodies are truncated to avoid increased memory usage issues under
