@@ -27,11 +27,13 @@ from scalyr_agent import scalyr_init
 
 scalyr_init()
 
+import sys
 import datetime
 import os
 import tempfile
 import threading
 import uuid
+import mock
 from mock import patch, MagicMock
 import six
 
@@ -173,136 +175,6 @@ class TestUtil(ScalyrTestCase):
         actual = scalyr_util.microseconds_since_epoch(dt)
         self.assertEquals(expected, actual)
 
-    def test_rfc3339_to_datetime(self):
-        s = "2015-08-06T14:40:56.123456Z"
-        expected = datetime.datetime(2015, 8, 6, 14, 40, 56, 123456)
-        actual = scalyr_util.rfc3339_to_datetime(s)
-
-        self.assertEquals(expected, actual)
-
-        s = "2015-08-06T14:40:56Z"
-        actual = scalyr_util.rfc3339_to_datetime(s)
-
-        self.assertEquals(datetime.datetime(2015, 8, 6, 14, 40, 56), actual)
-
-        s = "2015-08-06T14:40:56.123456"
-        actual = scalyr_util.rfc3339_to_datetime(s)
-        self.assertEquals(datetime.datetime(2015, 8, 6, 14, 40, 56), actual)
-
-    def test_rfc3339_to_datetime_truncated_nano(self):
-        s = "2015-08-06T14:40:56.123456789Z"
-        expected = datetime.datetime(2015, 8, 6, 14, 40, 56, 123456)
-        actual = scalyr_util.rfc3339_to_datetime(s)
-
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_datetime_bad_format_date_and_time_separator(self):
-        s = "2015-08-06 14:40:56.123456789Z"
-        expected = None
-        actual = scalyr_util.rfc3339_to_datetime(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_datetime_bad_format_has_timezone(self):
-        # currently this function only handles UTC.  Remove this test if
-        # updated to be more flexible
-        s = "2015-08-06T14:40:56.123456789+04:00"
-        expected = None
-        actual = scalyr_util.rfc3339_to_datetime(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch(self):
-        s = "2015-08-06T14:40:56.123456Z"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56, 123456)
-            )
-            * 1000
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-        s = "2015-08-06T14:40:56.123456"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56)
-            )
-            * 1000
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch_no_fractions(self):
-        s = "2015-08-06T14:40:56"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56)
-            )
-            * 1000
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-        s = "2015-08-06T14:40:56Z"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56)
-            )
-            * 1000
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch_some_fractions(self):
-        s = "2015-08-06T14:40:56.123Z"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56, 123000)
-            )
-            * 1000
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch_many_fractions(self):
-        s = "2015-08-06T14:40:56.123456789Z"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56, 123456)
-            )
-            * 1000
-            + 789
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch_too_many_fractions(self):
-        s = "2015-08-06T14:40:56.123456789999Z"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2015, 8, 6, 14, 40, 56, 123456)
-            )
-            * 1000
-            + 789
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch_strange_value(self):
-        s = "2017-09-20T20:44:00.123456Z"
-        expected = (
-            scalyr_util.microseconds_since_epoch(
-                datetime.datetime(2017, 9, 20, 20, 44, 00, 123456)
-            )
-            * 1000
-        )
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertEquals(expected, actual)
-
-    def test_rfc3339_to_nanoseconds_since_epoch_bad_format_has_timezone(self):
-        s = "2015-08-06T14:40:56.123456789+04:00"
-        actual = scalyr_util.rfc3339_to_nanoseconds_since_epoch(s)
-        self.assertIs(None, actual)
-
     def test_uuid(self):
         first = scalyr_util.create_unique_id()
         second = scalyr_util.create_unique_id()
@@ -411,6 +283,51 @@ class TestUtil(ScalyrTestCase):
             "attributes_parser",
         )
 
+    def test_get_web_url_from_upload_url(self):
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://agent.scalyr.com"),
+            "https://www.scalyr.com",
+        )
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://log.scalyr.com"),
+            "https://www.scalyr.com",
+        )
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://upload.scalyr.com"),
+            "https://www.scalyr.com",
+        )
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://app.scalyr.com"),
+            "https://www.scalyr.com",
+        )
+
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://agent.eu.scalyr.com"),
+            "https://www.eu.scalyr.com",
+        )
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://log.eu.scalyr.com"),
+            "https://www.eu.scalyr.com",
+        )
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://upload.eu.scalyr.com"),
+            "https://www.eu.scalyr.com",
+        )
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://app.eu.scalyr.com"),
+            "https://www.eu.scalyr.com",
+        )
+
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://logstaging.scalyr.com"),
+            "https://logstaging.scalyr.com",
+        )
+
+        self.assertEqual(
+            scalyr_util.get_web_url_from_upload_url("https://logstaging.eu.scalyr.com"),
+            "https://logstaging.eu.scalyr.com",
+        )
+
 
 class TestRateLimiter(ScalyrTestCase):
     def setUp(self):
@@ -605,8 +522,37 @@ class TestStoppableThread(ScalyrTestCase):
             self._run_counter += 1
             run_state.sleep_but_awaken_if_stopped(0.03)
 
+    def test_register_on_stop_callback(self):
+        self.callback_called = False
+
+        def fake_callback():
+            self.callback_called = True
+
+        run_state = scalyr_util.RunState()
+        run_state.register_on_stop_callback(fake_callback)
+        run_state.stop()
+        self.assertTrue(self.callback_called)
+
+    def test_remove_on_stop_callback(self):
+        self.callback_called = False
+
+        def fake_callback():
+            self.callback_called = True
+
+        run_state = scalyr_util.RunState()
+        run_state.register_on_stop_callback(fake_callback)
+        run_state.remove_on_stop_callback(fake_callback)
+        run_state.stop()
+        self.assertFalse(self.callback_called)
+
 
 class TestScriptEscalator(ScalyrTestCase):
+    def tearDown(self):
+        super(TestScriptEscalator, self).tearDown()
+
+        if "__main__" in sys.modules:
+            del sys.modules["__main__"]
+
     def test_is_user_change_required(self):
         (test_instance, controller) = self.create_instance("czerwin", "fileA", "steve")
         self.assertTrue(test_instance.is_user_change_required())
@@ -617,12 +563,17 @@ class TestScriptEscalator(ScalyrTestCase):
         self.assertFalse(test_instance.is_user_change_required())
 
     def test_change_user_and_rerun_script(self):
+        # NOTE: __main__.__file__ might not be set when running tests under pytests or nosetests
+        mock_main = mock.Mock()
+        mock_main.__file__ = "/tmp/file.py"
+        sys.modules["__main__"] = mock_main
+
         (test_instance, controller) = self.create_instance("czerwin", "fileA", "steve")
         self.assertEquals(test_instance.change_user_and_rerun_script("random"), 0)
 
         self.assertEquals(controller.call_count, 1)
         self.assertEquals(controller.last_call["user"], "steve")
-        self.assertIsNotNone(controller.last_call["script_file"])
+        self.assertEqual(controller.last_call["script_file"], "/tmp/file.py")
 
     def create_instance(self, current_user, config_file, config_owner):
         controller = TestScriptEscalator.ControllerMock(
