@@ -36,7 +36,6 @@ __author__ = "czerwin@scalyr.com"
 
 import logging
 import base64
-import calendar
 import datetime
 import os
 import threading
@@ -44,7 +43,6 @@ import time
 import uuid
 
 import scalyr_agent.json_lib as json_lib
-from scalyr_agent.compat import custom_any as any
 from scalyr_agent.json_lib import JsonParseException
 from scalyr_agent.platform_controller import CannotExecuteAsUser
 
@@ -66,6 +64,11 @@ except ImportError:
 
     new_md5 = False
 
+# Those imports have been moved in #494 so this alias is left here is place just in case for
+# backward compatibility reasons
+from scalyr_agent.date_parsing_utils import rfc3339_to_nanoseconds_since_epoch  # NOQA
+from scalyr_agent.date_parsing_utils import rfc3339_to_datetime  # NOQA
+
 
 USJON_NOT_AVAILABLE_MSG = """
 ujson library is not available. You can install it using pip:
@@ -90,6 +93,13 @@ Original error: %s
 """.strip()
 
 
+# True if json.dumps should sort the keys and use custom ident which doesn't include a whitespace
+# after a comma.
+# This option adds significant overhead so it's only used by the tests to make asserting on the
+# serialized values easier.
+SORT_KEYS = False
+
+
 def get_json_implementation(lib_name):
     if lib_name not in ["json", "ujson", "orjson"]:
         raise ValueError("Unsupported json library %s" % lib_name)
@@ -122,9 +132,9 @@ def get_json_implementation(lib_name):
                     "ujson does not correctly encode objects of type: %s" % type(obj)
                 )
             if fp is not None:
-                return ujson.dump(obj, fp, sort_keys=True)
+                return ujson.dump(obj, sort_keys=SORT_KEYS)
             else:
-                return ujson.dumps(obj, sort_keys=True)
+                return ujson.dumps(obj, sort_keys=SORT_KEYS)
 
         return lib_name, ujson_dumps_custom, ujson.loads
 
@@ -156,12 +166,16 @@ def get_json_implementation(lib_name):
             :return: If fp is not None, then the string representing the serialization.
             :rtype: Python3 - six.text_type, Python2 - six.binary_type
             """
+            if SORT_KEYS:
+                kwargs = {"sort_keys": True, "separators": (",", ":")}
+            else:
+                kwargs = {}
 
             if fp is not None:
                 # Eliminate spaces by default. Python 2.4 does not support partials.
-                return json.dump(obj, fp, sort_keys=True, separators=(",", ":"))
+                return json.dump(obj, fp, **kwargs)
             else:
-                return json.dumps(obj, sort_keys=True, separators=(",", ":"))
+                return json.dumps(obj, **kwargs)
 
         if sys.version_info[0] == 3 and sys.version_info[1] < 6:
             # wrap native json library 'loads' in Python3.5 and below, because it does not accept bytes.
@@ -186,19 +200,34 @@ def set_json_lib(lib_name):
     _json_lib, _json_encode, _json_decode = get_json_implementation(lib_name)
 
 
-try:
-    set_json_lib("ujson")
-except ImportError:
+# Set default json library we will use. We start with the most efficient and falling back to less
+# efficient library as a fallback.
+# We default to orjson under Python 3 (if available), since it's substantially faster than ujson for
+# encoding
+if six.PY3:
+    JSON_LIBS_TO_USE = ["orjson", "ujson", "json"]
+else:
+    JSON_LIBS_TO_USE = ["ujson", "json"]
+
+last_error = None
+for json_lib_to_use in JSON_LIBS_TO_USE:
     try:
-        set_json_lib("json")
-    except ImportError:
-        # Note, we cannot use a logger here because of dependency issues with this file and scalyr_logging.py
-        print(
-            "No default json library found which should be present in all Python >= 2.6.  "
-            "Python < 2.6 is not supported.  Exiting.",
-            file=sys.stderr,
-        )
-        sys.exit(1)
+        set_json_lib(json_lib_to_use)
+    except ImportError as e:
+        last_error = e
+    else:
+        last_error = None
+        break
+
+# Note, we cannot use a logger here because of dependency issues with this file and scalyr_logging.py
+if last_error:
+    # Note, we cannot use a logger here because of dependency issues with this file and scalyr_logging.py
+    print(
+        "No default json library found which should be present in all Python >= 2.6. "
+        "Python < 2.6 is not supported.  Exiting.",
+        file=sys.stderr,
+    )
+    sys.exit(1)
 
 
 def get_json_lib():
@@ -509,110 +538,6 @@ def seconds_since_epoch(date_time, epoch=None):
     return microseconds_since_epoch(date_time) / 10.0 ** 6
 
 
-def rfc3339_to_datetime(string):
-    """Returns a date time from a rfc3339 formatted timestamp.
-
-    We have to do some tricksy things to support python 2.4, which doesn't support
-    datetime.strptime or the fractional component %f in format strings
-
-    This doesn't do any complex testing and assumes the string is well formed
-    and in UTC (e.g. uses Z at the end rather than a time offset)
-
-    @param string: a date/time in rfc3339 format, e.g. 2015-08-03T09:12:43.143757463Z
-
-    @rtype datetime.datetime
-    """
-    # split the string in to main time and fractional component
-    parts = string.split(".")
-
-    # it's possible that the time does not have a fractional component
-    # e.g 2015-08-03T09:12:43Z, in this case 'parts' will only have a
-    # single element that should end in Z.  Strip the Z if it exists
-    # so we can use the same format string for processing the main
-    # date+time regardless of whether the time has a fractional component.
-    if parts[0].endswith("Z"):
-        parts[0] = parts[0][:-1]
-
-    # create a datetime object
-    try:
-        tm = time.strptime(parts[0], "%Y-%m-%dT%H:%M:%S")
-    except ValueError:
-        return None
-
-    dt = datetime.datetime(*(tm[0:6]))
-
-    # now add the fractional part
-    if len(parts) > 1:
-        fractions = parts[1]
-        # if we had a fractional component it should terminate in a Z
-        if not fractions.endswith("Z"):
-            # we don't handle non UTC timezones yet
-            if any(c in fractions for c in "+-"):
-                return None
-            return dt
-
-        # remove the Z and just process the fraction.
-        fractions = fractions[:-1]
-        to_micros = 6 - len(fractions)
-        micro = int(int(fractions) * 10 ** to_micros)
-        dt = dt.replace(microsecond=micro)
-
-    return dt
-
-
-def rfc3339_to_nanoseconds_since_epoch(string):
-    """Returns nanoseconds since the epoch from a rfc3339 formatted timestamp.
-
-    We have to do some tricksy things to support python 2.4, which doesn't support
-    datetime.strptime or the fractional component %f in format strings
-
-    This doesn't do any complex testing and assumes the string is well formed
-    and in UTC (e.g. uses Z at the end rather than a time offset)
-
-    @param string: a date/time in rfc3339 format, e.g. 2015-08-03T09:12:43.143757463Z
-
-    @rtype long
-    """
-    # split the string in to main time and fractional component
-    parts = string.split(".")
-
-    # it's possible that the time does not have a fractional component
-    # e.g 2015-08-03T09:12:43Z, in this case 'parts' will only have a
-    # single element that should end in Z.  Strip the Z if it exists
-    # so we can use the same format string for processing the main
-    # date+time regardless of whether the time has a fractional component.
-    if parts[0].endswith("Z"):
-        parts[0] = parts[0][:-1]
-
-    # create a datetime object
-    try:
-        tm = time.strptime(parts[0], "%Y-%m-%dT%H:%M:%S")
-    except ValueError:
-        return None
-
-    nano_seconds = int(calendar.timegm(tm[0:6])) * 1000000000
-    nanos = 0
-
-    # now add the fractional part
-    if len(parts) > 1:
-        fractions = parts[1]
-        # if the fractional part doesn't end in Z we likely have a
-        # malformed time, so just return the current value
-        if not fractions.endswith("Z"):
-            # we don't handle non UTC timezones yet
-            if any(c in fractions for c in "+-"):
-                return None
-
-            return nano_seconds
-
-        # strip the final 'Z' and use the final number for processing
-        fractions = fractions[:-1]
-        to_nanos = 9 - len(fractions)
-        nanos = int(int(fractions) * 10 ** to_nanos)
-
-    return nano_seconds + nanos
-
-
 def format_time(time_value):
     """Returns the time converted to a string in the common format used throughout the agent and in UTC.
 
@@ -698,6 +623,14 @@ def get_parser_from_config(base_config, attributes, default_parser):
     return default_parser
 
 
+def get_web_url_from_upload_url(server):
+    server = server.replace("https://agent.", "https://www.")
+    server = server.replace("https://log.", "https://www.")
+    server = server.replace("https://upload.", "https://www.")
+    server = server.replace("https://app.", "https://www.")
+    return server
+
+
 class JsonReadFileException(Exception):
     """Raised when a failure occurs when reading a file as a JSON object."""
 
@@ -1376,8 +1309,10 @@ class ScriptEscalator(object):
                 # noinspection PyUnresolvedReferences
                 import __main__  # type: ignore
 
-                script_file_path = __main__.__file__
-                if not os.path.isabs(script_file_path):
+                # NOTE: Under some scenarios when running in parallel with pytest __file__ attribute
+                # won't be set.
+                script_file_path = getattr(__main__, "__file__", None)
+                if script_file_path and not os.path.isabs(script_file_path):
                     script_file_path = os.path.normpath(
                         os.path.join(self.__cwd, script_file_path)
                     )
